#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const daemon_1 = __importDefault(require("@runapm/daemon"));
const cli_table3_1 = __importDefault(require("cli-table3"));
const enquirer_1 = require("enquirer");
const yargs_1 = __importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const main = async () => {
    void yargs_1.default(helpers_1.hideBin(process.argv))
        .command("list", "list running processes", async () => {
        const processes = await daemon_1.default.getProcessList();
        const table = new cli_table3_1.default({ head: ["PID", "Command", "Working Directory"] });
        table.push(...processes.map(({ pid, command: c }) => [
            pid,
            c.args.join(" "),
            c.cwd,
        ]));
        console.log(table.toString());
        await daemon_1.default.disconnect();
    })
        .command("restart", "restart command processes", async () => {
        const processes = await daemon_1.default.getProcessList();
        if (processes.length === 0) {
            console.log("No active processes");
        }
        else {
            const answers = await enquirer_1.prompt([
                {
                    type: "multiselect",
                    name: "pids",
                    message: "Select process to be restarted",
                    choices: processes.map((o) => ({
                        name: `${o.pid}: ${o.command.args.join(" ")}`,
                        value: o.pid.toString(),
                    })),
                },
            ]).catch(() => ({ pids: [] }));
            if (answers.pids.length > 0) {
                await daemon_1.default.restartProcesses(answers.pids.map((x) => Number.parseInt(x, 10)));
            }
        }
        await daemon_1.default.disconnect();
    })
        .command("shutdown", "shutdown daemon process", async () => {
        await daemon_1.default.shutdown();
        console.log("shutdown daemon");
    })
        .help().argv;
};
if (require.main === module)
    void main();
//# sourceMappingURL=daemon-cli.js.map