"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeArgv = void 0;
const run_script_1 = __importDefault(require("@npmcli/run-script"));
const daemon_1 = __importDefault(require("@runapm/daemon"));
const execa_1 = __importDefault(require("execa"));
const exit_1 = __importDefault(require("exit"));
const p_queue_1 = __importDefault(require("p-queue"));
const picomatch_1 = __importDefault(require("picomatch"));
const read_pkg_1 = __importDefault(require("read-pkg"));
const which_1 = __importDefault(require("which"));
const yargs_1 = __importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const resolveCommand = async (cmd) => {
    try {
        return await which_1.default(cmd);
    }
    catch { }
    return false;
};
class Executor {
    constructor(argv) {
        this.#argv = argv;
    }
    #argv;
    get argv() {
        return this.#argv;
    }
    get cmd() {
        return this.#argv._[0];
    }
    get cmdArgs() {
        return this.#argv._.slice(1);
    }
}
class CommandExecutor extends Executor {
    constructor() {
        super(...arguments);
        this.#queue = new p_queue_1.default({ concurrency: 1 });
    }
    #cp;
    #queue;
    async run() {
        const resolved = await resolveCommand(this.cmd);
        if (resolved === false) {
            console.log(`Unknown command: ${this.cmd}`);
            return 127;
        }
        void daemon_1.default.registerRestart(() => {
            const p = this.restart();
            void this.#queue.add(async () => p);
        });
        this.#cp = this.spawn();
        await this.#queue.add(async () => this.#cp);
        await this.#queue.onIdle();
        return this.#cp.exitCode ?? 0;
    }
    restart() {
        this.#cp?.kill();
        this.#cp = this.spawn();
        return this.#cp;
    }
    spawn() {
        return execa_1.default(this.cmd, this.cmdArgs, {
            stdio: "inherit",
            reject: false,
        });
    }
}
class ScriptsExecutor extends Executor {
    async run() {
        const pkg = await read_pkg_1.default();
        const cmds = this.argv._.map((s) => {
            const isMatch = picomatch_1.default(s);
            return Object.keys(pkg.scripts ?? {}).filter((k) => isMatch(k));
        }).flat();
        const runScriptOpts = {
            path: process.cwd(),
            stdio: "inherit",
            banner: false,
        };
        if (this.argv.p) {
            await Promise.all(cmds.map((event) => run_script_1.default({ ...runScriptOpts, event })));
        }
        else {
            for await (const event of cmds) {
                await run_script_1.default({ ...runScriptOpts, event });
            }
        }
        return 0;
    }
}
const normalizeArgv = (argv, argv0) => {
    let i = argv.findIndex((s) => s === argv0) + 2;
    while (i < argv.length) {
        if (argv[i] === "--") {
            return argv;
        }
        if (argv[i].startsWith("-")) {
            i += 1;
        }
        else {
            break;
        }
    }
    if (i === argv.length) {
        return argv;
    }
    const argvNew = [...argv];
    argvNew.splice(i, 0, "--");
    return argvNew;
};
exports.normalizeArgv = normalizeArgv;
const main = async () => {
    void yargs_1.default(helpers_1.hideBin(exports.normalizeArgv(process.argv, process.argv0)))
        .command("$0", "execute command", (yargs) => yargs.options({
        s: {
            describe: "Run npm-scripts sequentially",
            type: "boolean",
        },
        p: {
            describe: "Run npm-scripts in parallel",
            type: "boolean",
        },
    }), async (argv) => {
        const commandArgs = argv._;
        if (commandArgs.length === 0) {
            yargs_1.default.showHelp();
            return;
        }
        const notifyStartPromise = daemon_1.default.notifyProcessStart({
            args: commandArgs,
            cwd: process.cwd(),
        });
        const Executor = argv.s || argv.p ? ScriptsExecutor : CommandExecutor;
        const exitCode = await new Executor(argv).run();
        await notifyStartPromise;
        await daemon_1.default.notifyProcessEnd();
        await daemon_1.default.disconnect();
        exit_1.default(exitCode);
    })
        .help().argv;
};
exports.default = main;
//# sourceMappingURL=index.js.map